@@ -46,7 +46,7 @@ Codec <- R6Class("Codec", inherit = ArrowObject,
 )
 Codec$create <- function(type = "gzip", compression_level = NA) {
   if (is.string(type)) {
-    type <- unique_ptr(Codec, util___Codec__Create(
+    type <- shared_ptr(Codec, util___Codec__Create(
       compression_from_name(type), compression_level
     ))
   }
