@@ -42,12 +42,42 @@ DateUnit <- enum("DateUnit", DAY = 0L, MILLI = 1L)
 #' @rdname enums
 #' @export
 Type <- enum("Type::type",
-  "NA" = 0L, BOOL = 1L, UINT8 = 2L, INT8 = 3L, UINT16 = 4L, INT16 = 5L,
-  UINT32 = 6L, INT32 = 7L, UINT64 = 8L, INT64 = 9L,
-  HALF_FLOAT = 10L, FLOAT = 11L, DOUBLE = 12L, STRING = 13L,
-  BINARY = 14L, FIXED_SIZE_BINARY = 15L, DATE32 = 16L, DATE64 = 17L, TIMESTAMP = 18L,
-  TIME32 = 19L, TIME64 = 20L, INTERVAL = 21L, DECIMAL = 22L, LIST = 23L, STRUCT = 24L,
-  UNION = 25L, DICTIONARY = 26L, MAP = 27L
+  "NA" = 0L,
+  BOOL = 1L,
+  UINT8 = 2L,
+  INT8 = 3L,
+  UINT16 = 4L,
+  INT16 = 5L,
+  UINT32 = 6L,
+  INT32 = 7L,
+  UINT64 = 8L,
+  INT64 = 9L,
+  HALF_FLOAT = 10L,
+  FLOAT = 11L,
+  DOUBLE = 12L,
+  STRING = 13L,
+  BINARY = 14L,
+  FIXED_SIZE_BINARY = 15L,
+  DATE32 = 16L,
+  DATE64 = 17L,
+  TIMESTAMP = 18L,
+  TIME32 = 19L,
+  TIME64 = 20L,
+  INTERVAL_MONTHS = 21L,
+  INTERVAL_DAY_TIME = 22L,
+  DECIMAL = 23L,
+  LIST = 24L,
+  STRUCT = 25L,
+  SPARSE_UNION = 26L,
+  DENSE_UNION = 27L,
+  DICTIONARY = 28L,
+  MAP = 29L,
+  EXTENSION = 30L,
+  FIXED_SIZE_LIST = 31L,
+  DURATION = 32L,
+  LARGE_STRING = 33L,
+  LARGE_BINARY = 34L,
+  LARGE_LIST = 35L
 )
 
 #' @rdname enums
@@ -69,7 +99,7 @@ FileMode <- enum("FileMode",
 
 #' @rdname enums
 #' @export
-MessageType <- enum("Message::Type",
+MessageType <- enum("MessageType",
   NONE = 0L, SCHEMA = 1L, DICTIONARY_BATCH = 2L, RECORD_BATCH = 3L, TENSOR = 4L
 )
 
@@ -91,3 +121,9 @@ FileType <- enum("FileType",
 ParquetVersionType <- enum("ParquetVersionType",
   PARQUET_1_0 = 0L, PARQUET_2_0 = 1L
 )
+
+#' @export
+#' @rdname enums
+MetadataVersion <- enum("MetadataVersion",
+  V1 = 0L, V2 = 1L, V3 = 2L, V4 = 3L, V5 = 4L
+)
