@@ -17,48 +17,45 @@
 
 #include "./arrow_types.h"
 
-using Rcpp::LogicalVector;
-using Rcpp::no_init;
-
 #if defined(ARROW_R_WITH_ARROW)
 
 #include <arrow/array.h>
 #include <arrow/util/bitmap_reader.h>
 
-void arrow::r::validate_slice_offset(int offset, int len) {
+void arrow::r::validate_slice_offset(R_xlen_t offset, int64_t len) {
   if (offset == NA_INTEGER) {
-    Rcpp::stop("Slice 'offset' cannot be NA");
+    cpp11::stop("Slice 'offset' cannot be NA");
   }
   if (offset < 0) {
-    Rcpp::stop("Slice 'offset' cannot be negative");
+    cpp11::stop("Slice 'offset' cannot be negative");
   }
   if (offset > len) {
-    Rcpp::stop("Slice 'offset' greater than array length");
+    cpp11::stop("Slice 'offset' greater than array length");
   }
 }
 
-void arrow::r::validate_slice_length(int length, int available) {
+void arrow::r::validate_slice_length(R_xlen_t length, int64_t available) {
   if (length == NA_INTEGER) {
-    Rcpp::stop("Slice 'length' cannot be NA");
+    cpp11::stop("Slice 'length' cannot be NA");
   }
   if (length < 0) {
-    Rcpp::stop("Slice 'length' cannot be negative");
+    cpp11::stop("Slice 'length' cannot be negative");
   }
   if (length > available) {
-    Rcpp::warning("Slice 'length' greater than available length");
+    cpp11::warning("Slice 'length' greater than available length");
   }
 }
 
 // [[arrow::export]]
 std::shared_ptr<arrow::Array> Array__Slice1(const std::shared_ptr<arrow::Array>& array,
-                                            int offset) {
+                                            R_xlen_t offset) {
   arrow::r::validate_slice_offset(offset, array->length());
   return array->Slice(offset);
 }
 
 // [[arrow::export]]
 std::shared_ptr<arrow::Array> Array__Slice2(const std::shared_ptr<arrow::Array>& array,
-                                            int offset, int length) {
+                                            R_xlen_t offset, R_xlen_t length) {
   arrow::r::validate_slice_offset(offset, array->length());
   arrow::r::validate_slice_length(length, array->length() - offset);
   return array->Slice(offset, length);
@@ -66,21 +63,21 @@ std::shared_ptr<arrow::Array> Array__Slice2(const std::shared_ptr<arrow::Array>&
 
 void arrow::r::validate_index(int i, int len) {
   if (i == NA_INTEGER) {
-    Rcpp::stop("'i' cannot be NA");
+    cpp11::stop("'i' cannot be NA");
   }
   if (i < 0 || i >= len) {
-    Rcpp::stop("subscript out of bounds");
+    cpp11::stop("subscript out of bounds");
   }
 }
 
 // [[arrow::export]]
-bool Array__IsNull(const std::shared_ptr<arrow::Array>& x, int i) {
+bool Array__IsNull(const std::shared_ptr<arrow::Array>& x, R_xlen_t i) {
   arrow::r::validate_index(i, x->length());
   return x->IsNull(i);
 }
 
 // [[arrow::export]]
-bool Array__IsValid(const std::shared_ptr<arrow::Array>& x, int i) {
+bool Array__IsValid(const std::shared_ptr<arrow::Array>& x, R_xlen_t i) {
   arrow::r::validate_index(i, x->length());
   return x->IsValid(i);
 }
@@ -129,16 +126,16 @@ std::shared_ptr<arrow::ArrayData> Array__data(
 
 // [[arrow::export]]
 bool Array__RangeEquals(const std::shared_ptr<arrow::Array>& self,
-                        const std::shared_ptr<arrow::Array>& other, int start_idx,
-                        int end_idx, int other_start_idx) {
+                        const std::shared_ptr<arrow::Array>& other, R_xlen_t start_idx,
+                        R_xlen_t end_idx, R_xlen_t other_start_idx) {
   if (start_idx == NA_INTEGER) {
-    Rcpp::stop("'start_idx' cannot be NA");
+    cpp11::stop("'start_idx' cannot be NA");
   }
   if (end_idx == NA_INTEGER) {
-    Rcpp::stop("'end_idx' cannot be NA");
+    cpp11::stop("'end_idx' cannot be NA");
   }
   if (other_start_idx == NA_INTEGER) {
-    Rcpp::stop("'other_start_idx' cannot be NA");
+    cpp11::stop("'other_start_idx' cannot be NA");
   }
   return self->RangeEquals(*other, start_idx, end_idx, other_start_idx);
 }
@@ -149,22 +146,6 @@ std::shared_ptr<arrow::Array> Array__View(const std::shared_ptr<arrow::Array>& a
   return ValueOrStop(array->View(type));
 }
 
-// [[arrow::export]]
-LogicalVector Array__Mask(const std::shared_ptr<arrow::Array>& array) {
-  if (array->null_count() == 0) {
-    return LogicalVector(array->length(), true);
-  }
-
-  auto n = array->length();
-  LogicalVector res(no_init(n));
-  arrow::internal::BitmapReader bitmap_reader(array->null_bitmap()->data(),
-                                              array->offset(), n);
-  for (int64_t i = 0; i < n; i++, bitmap_reader.Next()) {
-    res[i] = bitmap_reader.IsSet();
-  }
-  return res;
-}
-
 // [[arrow::export]]
 void Array__Validate(const std::shared_ptr<arrow::Array>& array) {
   StopIfNotOk(array->Validate());
@@ -261,17 +242,17 @@ int64_t FixedSizeListArray__value_offset(
 }
 
 // [[arrow::export]]
-Rcpp::IntegerVector ListArray__raw_value_offsets(
+cpp11::writable::integers ListArray__raw_value_offsets(
     const std::shared_ptr<arrow::ListArray>& array) {
   auto offsets = array->raw_value_offsets();
-  return Rcpp::IntegerVector(offsets, offsets + array->length());
+  return cpp11::writable::integers(offsets, offsets + array->length());
 }
 
 // [[arrow::export]]
-Rcpp::IntegerVector LargeListArray__raw_value_offsets(
+cpp11::writable::integers LargeListArray__raw_value_offsets(
     const std::shared_ptr<arrow::LargeListArray>& array) {
   auto offsets = array->raw_value_offsets();
-  return Rcpp::IntegerVector(offsets, offsets + array->length());
+  return cpp11::writable::integers(offsets, offsets + array->length());
 }
 
 #endif
