@@ -17,9 +17,6 @@
 
 #include "./arrow_types.h"
 
-using Rcpp::List;
-using Rcpp::wrap;
-
 #if defined(ARROW_R_WITH_ARROW)
 #include <arrow/array/data.h>
 
@@ -45,8 +42,8 @@ int ArrayData__get_offset(const std::shared_ptr<arrow::ArrayData>& x) {
 }
 
 // [[arrow::export]]
-List ArrayData__buffers(const std::shared_ptr<arrow::ArrayData>& x) {
-  return wrap(x->buffers);
+cpp11::list ArrayData__buffers(const std::shared_ptr<arrow::ArrayData>& x) {
+  return cpp11::as_sexp(x->buffers);
 }
 
 #endif
