@@ -19,7 +19,7 @@
 
 #pragma once
 
-#include "./arrow_rcpp.h"
+#include "./arrow_cpp11.h"
 
 #if defined(ARROW_R_WITH_ARROW)
 #include <arrow/dataset/type_fwd.h>
@@ -29,6 +29,7 @@
 #include <arrow/status.h>
 #include <arrow/type_fwd.h>
 #include <arrow/util/compression.h>
+#include <arrow/util/value_parsing.h>
 
 namespace arrow {
 
@@ -83,15 +84,4 @@ class FileWriter;
 }  // namespace arrow
 }  // namespace parquet
 
-RCPP_EXPOSED_ENUM_NODECL(arrow::Type::type)
-RCPP_EXPOSED_ENUM_NODECL(arrow::DateUnit)
-RCPP_EXPOSED_ENUM_NODECL(arrow::ipc::MetadataVersion)
-RCPP_EXPOSED_ENUM_NODECL(arrow::TimeUnit::type)
-RCPP_EXPOSED_ENUM_NODECL(arrow::StatusCode)
-RCPP_EXPOSED_ENUM_NODECL(arrow::io::FileMode::type)
-RCPP_EXPOSED_ENUM_NODECL(arrow::ipc::MessageType)
-RCPP_EXPOSED_ENUM_NODECL(arrow::Compression::type)
-RCPP_EXPOSED_ENUM_NODECL(arrow::fs::FileType)
-RCPP_EXPOSED_ENUM_NODECL(parquet::ParquetVersion::type)
-
 #endif
