@@ -44,9 +44,11 @@ std::shared_ptr<arrow::Schema> RecordBatch__schema(
 
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatch> RecordBatch__ReplaceSchemaMetadata(
-    const std::shared_ptr<arrow::RecordBatch>& x, Rcpp::CharacterVector metadata) {
-  auto kv = std::shared_ptr<arrow::KeyValueMetadata>(new arrow::KeyValueMetadata(
-      metadata.names(), Rcpp::as<std::vector<std::string>>(metadata)));
+    const std::shared_ptr<arrow::RecordBatch>& x, cpp11::strings metadata) {
+  auto vec_metadata = cpp11::as_cpp<std::vector<std::string>>(metadata);
+  auto names_metadata = cpp11::as_cpp<std::vector<std::string>>(metadata.names());
+  auto kv = std::shared_ptr<arrow::KeyValueMetadata>(
+      new arrow::KeyValueMetadata(names_metadata, vec_metadata));
   return x->ReplaceSchemaMetadata(kv);
 }
 
@@ -63,7 +65,7 @@ arrow::ArrayVector RecordBatch__columns(
 
 // [[arrow::export]]
 std::shared_ptr<arrow::Array> RecordBatch__column(
-    const std::shared_ptr<arrow::RecordBatch>& batch, int i) {
+    const std::shared_ptr<arrow::RecordBatch>& batch, R_xlen_t i) {
   arrow::r::validate_index(i, batch->num_columns());
   return batch->column(i);
 }
@@ -75,9 +77,8 @@ std::shared_ptr<arrow::Array> RecordBatch__GetColumnByName(
 }
 
 // [[arrow::export]]
-std::shared_ptr<arrow::RecordBatch> RecordBatch__select(
-    const std::shared_ptr<arrow::RecordBatch>& batch,
-    const Rcpp::IntegerVector& indices) {
+std::shared_ptr<arrow::RecordBatch> RecordBatch__SelectColumns(
+    const std::shared_ptr<arrow::RecordBatch>& batch, cpp11::integers indices) {
   R_xlen_t n = indices.size();
   auto nrows = batch->num_rows();
 
@@ -85,7 +86,7 @@ std::shared_ptr<arrow::RecordBatch> RecordBatch__select(
   std::vector<std::shared_ptr<arrow::Array>> columns(n);
 
   for (R_xlen_t i = 0; i < n; i++) {
-    int pos = indices[i] - 1;
+    int pos = indices[i];
     fields[i] = batch->schema()->field(pos);
     columns[i] = batch->column(pos);
   }
@@ -94,24 +95,6 @@ std::shared_ptr<arrow::RecordBatch> RecordBatch__select(
   return arrow::RecordBatch::Make(schema, nrows, columns);
 }
 
-// [[arrow::export]]
-std::shared_ptr<arrow::RecordBatch> RecordBatch__from_dataframe(Rcpp::DataFrame tbl) {
-  Rcpp::CharacterVector names = tbl.names();
-
-  std::vector<std::shared_ptr<arrow::Field>> fields;
-  std::vector<std::shared_ptr<arrow::Array>> arrays;
-
-  for (int i = 0; i < tbl.size(); i++) {
-    SEXP x = tbl[i];
-    arrays.push_back(Array__from_vector(x, R_NilValue));
-    fields.push_back(
-        std::make_shared<arrow::Field>(std::string(names[i]), arrays[i]->type()));
-  }
-  auto schema = std::make_shared<arrow::Schema>(std::move(fields));
-
-  return arrow::RecordBatch::Make(schema, tbl.nrow(), std::move(arrays));
-}
-
 // [[arrow::export]]
 bool RecordBatch__Equals(const std::shared_ptr<arrow::RecordBatch>& self,
                          const std::shared_ptr<arrow::RecordBatch>& other,
@@ -121,23 +104,23 @@ bool RecordBatch__Equals(const std::shared_ptr<arrow::RecordBatch>& self,
 
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatch> RecordBatch__RemoveColumn(
-    const std::shared_ptr<arrow::RecordBatch>& batch, int i) {
+    const std::shared_ptr<arrow::RecordBatch>& batch, R_xlen_t i) {
   arrow::r::validate_index(i, batch->num_columns());
   return ValueOrStop(batch->RemoveColumn(i));
 }
 
 // [[arrow::export]]
 std::string RecordBatch__column_name(const std::shared_ptr<arrow::RecordBatch>& batch,
-                                     int i) {
+                                     R_xlen_t i) {
   arrow::r::validate_index(i, batch->num_columns());
   return batch->column_name(i);
 }
 
 // [[arrow::export]]
-Rcpp::CharacterVector RecordBatch__names(
+cpp11::writable::strings RecordBatch__names(
     const std::shared_ptr<arrow::RecordBatch>& batch) {
   int n = batch->num_columns();
-  Rcpp::CharacterVector names(n);
+  cpp11::writable::strings names(n);
   for (int i = 0; i < n; i++) {
     names[i] = batch->column_name(i);
   }
@@ -146,31 +129,31 @@ Rcpp::CharacterVector RecordBatch__names(
 
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatch> RecordBatch__Slice1(
-    const std::shared_ptr<arrow::RecordBatch>& self, int offset) {
+    const std::shared_ptr<arrow::RecordBatch>& self, R_xlen_t offset) {
   arrow::r::validate_slice_offset(offset, self->num_rows());
   return self->Slice(offset);
 }
 
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatch> RecordBatch__Slice2(
-    const std::shared_ptr<arrow::RecordBatch>& self, int offset, int length) {
+    const std::shared_ptr<arrow::RecordBatch>& self, R_xlen_t offset, R_xlen_t length) {
   arrow::r::validate_slice_offset(offset, self->num_rows());
   arrow::r::validate_slice_length(length, self->num_rows() - offset);
   return self->Slice(offset, length);
 }
 
 // [[arrow::export]]
-Rcpp::RawVector ipc___SerializeRecordBatch__Raw(
+cpp11::raws ipc___SerializeRecordBatch__Raw(
     const std::shared_ptr<arrow::RecordBatch>& batch) {
   // how many bytes do we need ?
   int64_t size;
   StopIfNotOk(arrow::ipc::GetRecordBatchSize(*batch, &size));
 
   // allocate the result raw vector
-  Rcpp::RawVector out(Rcpp::no_init(size));
+  cpp11::writable::raws out(size);
 
   // serialize into the bytes of the raw vector
-  auto buffer = std::make_shared<arrow::r::RBuffer<RAWSXP, Rcpp::RawVector>>(out);
+  auto buffer = std::make_shared<arrow::r::RBuffer<cpp11::raws>>(out);
   arrow::io::FixedSizeBufferWriter stream(buffer);
   StopIfNotOk(arrow::ipc::SerializeRecordBatch(
       *batch, arrow::ipc::IpcWriteOptions::Defaults(), &stream));
@@ -185,6 +168,7 @@ std::shared_ptr<arrow::RecordBatch> ipc___ReadRecordBatch__InputStream__Schema(
     const std::shared_ptr<arrow::Schema>& schema) {
   // TODO: promote to function arg
   arrow::ipc::DictionaryMemo memo;
+  StopIfNotOk(memo.fields().AddSchemaFields(*schema));
   return ValueOrStop(arrow::ipc::ReadRecordBatch(
       schema, &memo, arrow::ipc::IpcReadOptions::Defaults(), stream.get()));
 }
@@ -237,37 +221,22 @@ std::shared_ptr<arrow::RecordBatch> RecordBatch__from_arrays__known_schema(
   StopIfNotOk(arrow::r::count_fields(lst, &num_fields));
 
   if (schema->num_fields() != num_fields) {
-    Rcpp::stop("incompatible. schema has %d fields, and %d arrays are supplied",
-               schema->num_fields(), num_fields);
+    cpp11::stop("incompatible. schema has %d fields, and %d arrays are supplied",
+                schema->num_fields(), num_fields);
   }
 
   // convert lst to a vector of arrow::Array
   std::vector<std::shared_ptr<arrow::Array>> arrays(num_fields);
-  SEXP names = Rf_getAttrib(lst, R_NamesSymbol);
 
-  auto fill_array = [&arrays, &schema](int j, SEXP x, SEXP name) {
-    name = Rf_mkCharCE(Rf_translateCharUTF8(name), CE_UTF8);
-    if (schema->field(j)->name() != CHAR(name)) {
-      Rcpp::stop("field at index %d has name '%s' != '%s'", j + 1,
-                 schema->field(j)->name(), CHAR(name));
+  auto fill_array = [&arrays, &schema](int j, SEXP x, std::string name) {
+    if (schema->field(j)->name() != name) {
+      cpp11::stop("field at index %d has name '%s' != '%s'", j + 1,
+                  schema->field(j)->name().c_str(), name.c_str());
     }
     arrays[j] = arrow::r::Array__from_vector(x, schema->field(j)->type(), false);
   };
 
-  for (R_xlen_t i = 0, j = 0; j < num_fields; i++) {
-    SEXP name_i = STRING_ELT(names, i);
-    SEXP x_i = VECTOR_ELT(lst, i);
-
-    if (LENGTH(name_i) == 0) {
-      SEXP names_x_i = Rf_getAttrib(x_i, R_NamesSymbol);
-      for (R_xlen_t k = 0; k < XLENGTH(x_i); k++, j++) {
-        fill_array(j, VECTOR_ELT(x_i, k), STRING_ELT(names_x_i, k));
-      }
-    } else {
-      fill_array(j, x_i, name_i);
-      j++;
-    }
-  }
+  arrow::r::TraverseDots(lst, num_fields, fill_array);
 
   int64_t num_rows = 0;
   StopIfNotOk(arrow::r::check_consistent_array_size(arrays, &num_rows));
@@ -280,7 +249,7 @@ namespace r {
 arrow::Status CollectRecordBatchArrays(
     SEXP lst, const std::shared_ptr<arrow::Schema>& schema, int num_fields, bool inferred,
     std::vector<std::shared_ptr<arrow::Array>>& arrays) {
-  auto extract_one_array = [&arrays, &schema, inferred](int j, SEXP x, SEXP name) {
+  auto extract_one_array = [&arrays, &schema, inferred](int j, SEXP x, cpp11::r_string) {
     arrays[j] = arrow::r::Array__from_vector(x, schema->field(j)->type(), inferred);
   };
   arrow::r::TraverseDots(lst, num_fields, extract_one_array);
