@@ -38,6 +38,12 @@ expect_equivalent <- function(object, expected, ...) {
 
 # expect_equal but for DataTypes, so the error prints better
 expect_type_equal <- function(object, expected, ...) {
+  if (is.Array(object)) {
+    object <- object$type
+  }
+  if (is.Array(expected)) {
+    expected <- expected$type
+  }
   expect_equal(object, expected, ..., label = object$ToString(), expected.label = expected$ToString())
 }
 
@@ -46,3 +52,10 @@ expect_match_arg_error <- function(object, values=c()) {
 }
 
 expect_deprecated <- expect_warning
+
+verify_output <- function(...) {
+  if (isTRUE(grepl("conda", R.Version()$platform))) {
+    skip("On conda")
+  }
+  testthat::verify_output(...)
+}
