@@ -408,6 +408,6 @@ test_that("FixedSizeBinary", {
   # input validation
   expect_error(fixed_size_binary(NA), "'byte_width' cannot be NA")
   expect_error(fixed_size_binary(-1), "'byte_width' must be > 0")
-  expect_error(fixed_size_binary("four"), class = "Rcpp::not_compatible")
-  expect_error(fixed_size_binary(c(2, 4)), class = "Rcpp::not_compatible")
+  expect_error(fixed_size_binary("four"))
+  expect_error(fixed_size_binary(c(2, 4)))
 })
