@@ -80,10 +80,14 @@ test_that("SubTreeFilesystem", {
   DESCRIPTION <- system.file("DESCRIPTION", package = "arrow")
   file.copy(DESCRIPTION, file.path(td, "DESCRIPTION"))
 
-  local_fs <- LocalFileSystem$create()
-  st_fs <- SubTreeFileSystem$create(td, local_fs)
+  st_fs <- SubTreeFileSystem$create(td)
   expect_is(st_fs, "SubTreeFileSystem")
   expect_is(st_fs, "FileSystem")
+  expect_is(st_fs$base_fs, "LocalFileSystem")
+
+  # FIXME windows has a trailing slash for one but not the other
+  # expect_identical(normalizePath(st_fs$base_path), normalizePath(td))
+
   st_fs$CreateDir("test")
   st_fs$CopyFile("DESCRIPTION", "DESC.txt")
   infos <- st_fs$GetFileInfo(c("DESCRIPTION", "test", "nope", "DESC.txt"))
@@ -93,6 +97,7 @@ test_that("SubTreeFilesystem", {
   expect_equal(infos[[4L]]$type, FileType$File)
   expect_equal(infos[[4L]]$extension(), "txt")
 
+  local_fs <- LocalFileSystem$create()
   local_fs$DeleteDirContents(td)
   infos <- st_fs$GetFileInfo(c("DESCRIPTION", "test", "nope", "DESC.txt"))
   expect_equal(infos[[1L]]$type, FileType$NotFound)
@@ -129,6 +134,14 @@ test_that("FileSystem$from_uri", {
   skip_if_not_available("s3")
   fs_and_path <- FileSystem$from_uri("s3://ursa-labs-taxi-data")
   expect_is(fs_and_path$fs, "S3FileSystem")
+  expect_identical(fs_and_path$fs$region, "us-east-2")
+})
+
+test_that("SubTreeFileSystem$create() with URI", {
+  skip_on_cran()
+  skip_if_not_available("s3")
+  fs <- SubTreeFileSystem$create("s3://ursa-labs-taxi-data")
+  expect_is(fs, "SubTreeFileSystem")
 })
 
 test_that("S3FileSystem", {
@@ -137,3 +150,14 @@ test_that("S3FileSystem", {
   s3fs <- S3FileSystem$create()
   expect_is(s3fs, "S3FileSystem")
 })
+
+test_that("s3_bucket", {
+  skip_on_cran()
+  skip_if_not_available("s3")
+  bucket <- s3_bucket("ursa-labs-r-test")
+  expect_is(bucket, "SubTreeFileSystem")
+  expect_is(bucket$base_fs, "S3FileSystem")
+  expect_identical(bucket$region, "us-west-2")
+  skip_on_os("windows") # FIXME
+  expect_identical(bucket$base_path, "ursa-labs-r-test/")
+})
